#include <stdio.h>
#include "glincludes.h"
#include "Trackball.h"
#include "shaderdev.h"


// Burkhard's trackball at work again
CTrackball trackball;
float fov = 45.0;
float aRatio = 1;

//Light values
const float ltSpecular[4] = {1.0f,1.0f,1.0f,1.0f};
const float ltAmbient[4] = {0.1f,0.1f,0.1f,1.0f};
const float ltDiffuse[4] = {0.8f,0.8f,0.8f,1.0f};
const float ltPosition[4] = {0.0f,0.0f,5.0f,1.0f};

/* Copied from 372A2 (2002)*/
void handleMouseMotion(int x, int y)
{	
	trackball.tbMotion(x, y);
}

/* Copied from 372A2 (2002)*/
void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

/* Copied from 372A2 (2002)*/
void handleKeyboardEvent(unsigned char key, int x, int y)
{
	switch(key){
		case 'x':
		case 'y':
		case 'z':
			trackball.tbKeyboard(key);
			break;
		case '0':
			selectShader(0);
			break;
		case '1':
			selectShader(1);
			break;
		case '2':
			selectShader(2);
			break;
		case '3':
			selectShader(3);
			break;
		case '4':
			selectShader(4);
			break;
		case '5':
			selectShader(5);
			break;
		case '6':
			selectShader(6);
			break;
		case '7':
			selectShader(7);
			break;
		case '8':
			selectShader(8);
			break;
		case '9':
			selectShader(9);
			break;
		case 'r':
		case 'R':
			loadShaders();
			break;
		case '-':
			fov += 5;
			if(fov > 150) fov = 150;
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(fov, aRatio, 5.0, 55.0);
			break;
		case '=':
			fov -= 5;
			if(fov < 0) fov = 0;
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(fov, aRatio, 5.0, 55.0);
			break;
		default:
			break;
	}
	// remember to draw new position
	glutPostRedisplay();
	
}

void display(void)
{
	// clear all pixels in frame buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	// set up matrix and affect changes made via the trackball
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();	
	gluLookAt(0,0,20, 0,0,0, 0,1,0);
    trackball.tbMatrix();

	glEnable(GL_DEPTH_TEST);
	
	drawShader();

	glFlush ();
	glutSwapBuffers();
}

/* Copied from 372A2 (2002)*/
void reshape(int width, int height ) {
	glViewport(0, 0, width, height);  // Largest possible square
	trackball.tbReshape(width, height);
	
	aRatio = (GLfloat) width/(GLfloat) height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, aRatio, 5.0, 55.0);
}

void init (int width, int height) 
{
	loadResources();
	// select clearing color (for glClear)
	glClearColor (0.0, 0.0, 0.0, 0.0);

	// initialize view (simple orthographic projection)
	aRatio = (GLfloat) width/(GLfloat) height;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(fov, aRatio, 5.0, 55.0);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// initialize lighting
	glLightfv(GL_LIGHT0, GL_SPECULAR, ltSpecular);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ltAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, ltDiffuse);
	glLightfv(GL_LIGHT0, GL_POSITION, ltPosition);

}

int main (int argc, char * argv[]) {
	const int width = 800;
	const int height = 800;
    glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize(width, height);
	glutCreateWindow("GLSL-TestApp");
	  
	GLenum err = glewInit();
	if (GLEW_OK != err){
		printf("Error initalizing GLEW\n");
	}
	
    init(width,height);
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
    glutKeyboardFunc(handleKeyboardEvent);
    glutDisplayFunc(display);
    glutReshapeFunc(reshape);
	
	
    glutMainLoop();
	
    return 0;
}
